@@ -21,10 +21,15 @@ import requests
 
 from sushy import exceptions
 from sushy.resources.task_monitor import TaskMonitor
+from sushy import utils
 
 LOG = logging.getLogger(__name__)
 
 
+_SERVER_SIDE_RETRIES = 5
+_SERVER_SIDE_RETRY_DELAY = 3
+
+
 class Connector(object):
 
     def __init__(
@@ -67,7 +72,8 @@ class Connector(object):
         self._session.close()
 
     def _op(self, method, path='', data=None, headers=None, blocking=False,
-            timeout=60, **extra_session_req_kwargs):
+            timeout=60, server_side_retries=_SERVER_SIDE_RETRIES,
+            **extra_session_req_kwargs):
         """Generic RESTful request handler.
 
         :param method: The HTTP method to be used, e.g: GET, POST,
@@ -87,15 +93,20 @@ class Connector(object):
         url = path if urlparse.urlparse(path).netloc else urlparse.urljoin(
             self._url, path)
         headers = headers or {}
-        if not any(k.lower() == 'odata-version' for k in headers):
+        lc_headers = [k.lower() for k in headers]
+        if data is not None and 'content-type' not in lc_headers:
+            headers['Content-Type'] = 'application/json'
+        if 'odata-version' not in lc_headers:
             headers['OData-Version'] = '4.0'
         # TODO(lucasagomes): We should mask the data to remove sensitive
         # information
         LOG.debug('HTTP request: %(method)s %(url)s; headers: %(headers)s; '
                   'body: %(data)s; blocking: %(blocking)s; timeout: '
                   '%(timeout)s; session arguments: %(session)s;',
-                  {'method': method, 'url': url, 'headers': headers,
-                   'data': data, 'blocking': blocking, 'timeout': timeout,
+                  {'method': method, 'url': url,
+                   'headers': utils.sanitize(headers),
+                   'data': utils.sanitize(data),
+                   'blocking': blocking, 'timeout': timeout,
                    'session': extra_session_req_kwargs})
         try:
             response = self._session.request(method, url, json=data,
@@ -113,16 +124,42 @@ class Connector(object):
         # Attempt to re-establish a session.
         try:
             exceptions.raise_for_response(method, url, response)
-        except exceptions.AccessError:
+        except exceptions.AccessError as e:
             if self._auth.can_refresh_session():
-                self._auth.refresh_session()
+                try:
+                    self._auth.refresh_session()
+                except exceptions.AccessError as refresh_exc:
+                    LOG.error("A failure occured while attempting to refresh "
+                              "the session. Error: %s", refresh_exc.message)
+                    raise
                 LOG.debug("Authentication refreshed successfully, "
                           "retrying the call.")
-                response = self._session.request(method, url, json=data,
-                                                 headers=headers,
-                                                 **extra_session_req_kwargs)
+                try:
+                    response = self._session.request(
+                        method, url, json=data,
+                        headers=headers,
+                        **extra_session_req_kwargs)
+                except exceptions.HTTPError as retry_exc:
+                    LOG.error("Failure occured while attempting to retry "
+                              "request after refreshing the session. Error: "
+                              "%s", retry_exc.message)
+                    raise
             else:
+                LOG.error("Authentication error detected. Cannot proceed: "
+                          "%s", e.message)
                 raise
+        except exceptions.ServerSideError as e:
+            if method.lower() != 'get' or server_side_retries <= 0:
+                raise
+            else:
+                LOG.warning('Got server side error %s in response to a '
+                            'GET request, retrying after %d seconds',
+                            e, _SERVER_SIDE_RETRY_DELAY)
+                time.sleep(_SERVER_SIDE_RETRY_DELAY)
+                return self._op(method, path, data=data, headers=headers,
+                                blocking=blocking, timeout=timeout,
+                                server_side_retries=server_side_retries - 1,
+                                **extra_session_req_kwargs)
 
         if blocking and response.status_code == 202:
             if not response.headers.get('location'):
@@ -146,6 +183,7 @@ class Connector(object):
                             'timeout': timeout})
                     raise exceptions.ConnectionError(url=url, error=m)
             response = mon.response
+            exceptions.raise_for_response(method, url, response)
 
         LOG.debug('HTTP response for %(method)s %(url)s: '
                   'status code: %(code)s',
